//+------------------------------------------------------------------+
//|                                 Smoothed adaptive stochastic.mq4 |
//|                                                           mladen |
//|                                                                  |
//| source : www.guppytraders.com                                    |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers    2
#property indicator_color1     Green
#property indicator_color2     Red
#property indicator_minimum    0
#property indicator_maximum    100
#property indicator_level1     80
#property indicator_level2     20
#property indicator_levelcolor DarkSlateGray
#property indicator_style2     STYLE_DOT

//
//
//
//
//

extern int VolatilityLookBack = 20;
extern int KSmoothing         =  3;
extern int DSmoothing         =  3;
extern int MaxLength          = 28;
extern int MinLength          =  7;

//
//
//
//
//

double StochK[];
double StochKr[];
double StochD[];
double stdDev[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,StochK);
   SetIndexBuffer(1,StochD);
   SetIndexBuffer(2,StochKr);
   SetIndexBuffer(3,stdDev);
   IndicatorShortName("Smoothed adaptive stochastic ("+VolatilityLookBack+","+KSmoothing+","+DSmoothing+","+MaxLength+","+MinLength+")");
   return(0);
}
int deinit() { return(0); }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;
   
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = Bars-counted_bars;

   //
   //
   //
   //
   //
    
   double alphak = 2.0 / (1.0+KSmoothing);
   double alphad = 2.0 / (1.0+DSmoothing);
   
   //
   //
   //
   //
   //
   
   for (i=limit; i>=0; i--)
   {  
      stdDev[i] = iStdDev(NULL,0,VolatilityLookBack,0,MODE_SMA,PRICE_CLOSE,i);
      
         double hdev  = stdDev[ArrayMaximum(stdDev,VolatilityLookBack,i)];
         double ldev  = stdDev[ArrayMinimum(stdDev,VolatilityLookBack,i)];
         double ratio = 0;
            if (hdev != ldev)
                ratio = (stdDev[i]-ldev)/(hdev-ldev);
         
         //
         //
         //
         //
         //
         
         int    currentLength = MathRound(MinLength+(MaxLength-MinLength)*(1.0-ratio));
         double high          = High[iHighest(NULL,0,MODE_HIGH,currentLength,i)];
         double low           =  Low[ iLowest(NULL,0,MODE_LOW ,currentLength,i)];
         double price         = iMA(NULL,0,1,0,MODE_SMA,PRICE_CLOSE,i);
         
            StochKr[i] = 0;
               if (high != low) StochKr[i] = 100.00*(price-low)/(high-low);
                                StochK[i]  = StochK[i+1] + alphak*(StochKr[i]-StochK[i+1]);
                                StochD[i]  = StochD[i+1] + alphad*(StochK[i] -StochD[i+1]);
   }

   //
   //
   //
   //
   //
   
   return(0);
}